/*****************************************************************************
 * 
 * mui-gtk - a wrapper library to wrap GTK+ calls to MUI
 *
 * Copyright (C) 2005 Oliver Brunner
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Contact information:
 *
 * Oliver Brunner
 *
 * E-Mail: mui-gtk "at" oliver-brunner.de
 *
 * $Id:
 *
 *****************************************************************************/

/* TODO:
 * - still a lot ;)
 */

#include <stdio.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <proto/exec.h>
#include <mui.h>

/* cross platform hooks */
#ifndef __AROS__
  #define MGTK_HOOK_OBJECT_POINTER Object *
  #define MGTK_HOOK_APTR APTR
  #define MGTK_USERFUNC_INIT
  #define MGTK_USERFUNC_EXIT
  #include <SDI_hook.h>
#else
  #include "mgtk_aros.h"
#endif

#include "debug.h"
#include "gtk.h"
#include "gtk_globals.h"

#include "classes/classes.h"

/*************************/
/* Hook/Callback mapping */
/*************************/

typedef struct _mgtkHookList mgtkHookList;
struct _mgtkHookList {
  Object *object;
  GCallback callback;
  gpointer data;
  int GConnectFlags;
  GtkWidget *widget;
  mgtkHookList *next;
};

mgtkHookList *mgtkhooklist=NULL;
mgtkHookList *mgtkhooklist_activated=NULL;
mgtkHookList *mgtkhooklist_value_changed=NULL;
mgtkHookList *mgtkhooklist_destroy=NULL;
  
/* call the gtk hook(s) of the supplied object 
 *       - multiple hooks per object are supported
 * TODO: - no parameters are passed!!
 */
void  call_gtk_hooks(Object *obj) {
  void (*callme)(GtkWidget *widget,gpointer data);
  mgtkHookList *i;
  mgtkHookList *found=NULL;

  i=mgtkhooklist;
  while(i!=NULL) {
    DebOut("compare %lx == %lx\n",i->object,obj);
    if(i->object == obj) {
      found=i; /* HERE IT IS!!*/
      DebOut("found callback: listitem: %lx callback %lx\n",i,i->callback);
      callme=(APTR) found->callback;
      /*call gtk hook */
      DebOut("calling: %lx(%s)\n",callme,found->data);
      if(found->GConnectFlags == G_CONNECT_SWAPPED) {
				(*callme)(found->data,found->widget);
      }
      else {
				(*callme)(found->widget,found->data);
      }
    }
    i=(APTR) i->next;
  }
  DebOut("end of scan\n");

  if(found==NULL) {
    printf("WARNING: call_gtk_hooks: unknown hook!?\n");
  }

  return;
}

void  call_gtk_hooks_activated(Object *obj) {
  void (*callme)(GtkWidget *widget,gpointer data);
  mgtkHookList *i;
  mgtkHookList *found=NULL;

  DebOut("call_gtk_hooks_activated\n");
  i=mgtkhooklist_activated;
  while(i!=NULL) {
    DebOut("compare %lx == %lx\n",i->object,obj);
    if(i->object == obj) {
      found=i; /* HERE IT IS!!*/
      DebOut("found callback: listitem: %lx callback %lx\n",i,i->callback);
      callme=(APTR) found->callback;
      /*call gtk hook */
      DebOut("calling: %lx(%s)\n",callme,found->data);
      if(found->GConnectFlags == G_CONNECT_SWAPPED) {
	(*callme)(found->data,found->widget);
      }
      else {
	(*callme)(found->widget,found->data);
      }
    }
    i=(APTR) i->next;
  }
  DebOut("end of scan\n");

  if(found==NULL) {
    printf("WARNING: call_gtk_hooks_activated: unknown hook!?\n");
  }

  return;
}

/*****************************************
 * mgtk_collhooks
 *
 * call all hooks match obj
 *****************************************/
void mgtk_call_hooks(mgtkHookList *hooklist,Object *obj) {
  void (*callme)(GtkWidget *widget,gpointer data);
  mgtkHookList *i;
  mgtkHookList *found=NULL;

  DebOut("mgtk_call_hooks(mgtkHookList %lx,Object %lx)\n",hooklist,obj);

  i=hooklist;

  while(i!=NULL) {
    DebOut("    i->object=%lx\n",i->object);
    if(i->object == obj) {
      found=i;
      DebOut("  found callback: listitem: %lx callback %lx\n",i,i->callback);
      callme=(APTR) found->callback;
      /*call gtk hook */
      DebOut("  calling: %lx(%s)\n",callme,found->data);
      if(found->GConnectFlags == G_CONNECT_SWAPPED) {
	(*callme)(found->data,found->widget);
      }
      else {
	(*callme)(found->widget,found->data);
      }
    }
    i=(APTR) i->next;
  }
  DebOut("  end of scan\n");

  if(found==NULL) {
    printf("WARNING: mgtk_call_hooks: unknown hook object (%x)!?\n",(int) obj);
  }
  return;
}


/*****************************************************
 * "value_changed"
 *****************************************************/
void  call_gtk_hooks_value_changed(Object *obj) {

  mgtk_call_hooks(mgtkhooklist_value_changed,obj);

  return;
}

HOOKPROTO(MUIHookFunc_value_changed,ULONG,MGTK_HOOK_OBJECT_POINTER obj,MGTK_HOOK_APTR bla)
{
	MGTK_USERFUNC_INIT
  DebOut("MUIHookFunc_value_changed called for obj %lx (%lx)\n",obj,bla);
  call_gtk_hooks_value_changed(obj);
  return(0);
	MGTK_USERFUNC_EXIT
}
MakeHook(MyMuiHook_value_changed, MUIHookFunc_value_changed);


HOOKPROTO(MUIHookFunc,ULONG,MGTK_HOOK_OBJECT_POINTER obj,MGTK_HOOK_APTR  bla)
{
	MGTK_USERFUNC_INIT
  DebOut("MUIHookFunc called for obj %lx (%lx)\n",obj,bla);
  call_gtk_hooks(obj);
  return(0);
	MGTK_USERFUNC_EXIT
}
MakeHook(MyMuiHook, MUIHookFunc);

HOOKPROTO(MUIHookFunc_activated,ULONG,MGTK_HOOK_OBJECT_POINTER obj,MGTK_HOOK_APTR bla)
{
	MGTK_USERFUNC_INIT
  DebOut("MUIHookFunc_activated called for obj %lx (%lx)\n",obj,bla);
  call_gtk_hooks_activated(obj);
  return(0);
	MGTK_USERFUNC_EXIT
}
MakeHook(MyMuiHook_activated, MUIHookFunc_activated);

mgtkHookList *mgtk_hooklist_item_new() {

  mgtkHookList *ret;

  ret=g_new0(mgtkHookList,1);
  return(ret);
}

void mgtk_hooklist_item_append(mgtkHookList **mgtkhooklist,mgtkHookList *ret) {

  mgtkHookList *i;

  DebOut("mgtk_hooklist_item_append(mgtkHookList %lx,mgtkHookList %lx)\n",mgtkhooklist,ret);

  if(*mgtkhooklist!=NULL) {
    i=*mgtkhooklist;
    DebOut("  scan list:\n");
    while(i->next != NULL) {
      DebOut("i=%lx\n",i);
      i=(APTR) i->next;
  }
    /*append*/
    i->next=(APTR) ret;
    DebOut("  append %lx to %lx\n",ret,i);
  }
  else {
    DebOut("  first in list: %lx\n",ret);
    *mgtkhooklist=ret;
  }
  return;
}

  /* argl, data is tricky..?
    * if you get a string as parameter, it will not be a valid pointer,
    * if the callback gets called :(. Same goes for all other data (eventually).
    * How can we fix this?
    * As a work around, we copy the string and remember, that it does not
    * work for others structs :(
    */
gpointer mgtk_prepare_data(gpointer data) {
  LONG *badguy;
  gchar *s;

  DebOut("mgtk_prepare_data(%lx)\n",data);

  /* check for NULL */
  if(data == (gpointer) NULL) {
    DebOut("  Hook parameter is a NULL pointer\n");
    return(data);
  }

  badguy=data;
  /* check for bad magic */

  if((*badguy) == 0xDEADBEEF ) {
    DebOut("  Hook parameter seems to be a struct\n");
    DebOut("  data: %lx \n",data);
    return(data);
  }
  else {
    DebOut("  Magic(?): %lx\n",data);

    /* string */
    DebOut("  Hook parameter seems to be a string!!\n");
    s=g_strdup(data);

    DebOut("  data: %s (now: %s)\n",data,s);
    return(s);
  }
}

/*******************************************
 *mgtk_get_action_object(GtkWidget *widget)
 *
 * in case a MuiObject is packed in a frame or such,
 * this function returns the action object
 * (frame with a checkbox -> checkbox)
 *******************************************/
Object *mgtk_get_mui_action_object(GtkWidget *widget) {

  if(widget->nextObject!=NULL) {
    return((APTR) widget->nextObject);
  }
  if(widget->MuiObject==NULL) {
    printf("ERROR: mgtk_get_mui_action_object called on NULL pointer..\n");
  }
  return(widget->MuiObject);
}
 
/***************************************************************************/

ULONG muiaction_activate[IS_LASTONE];
ULONG muiaction_clicked[IS_LASTONE];
ULONG muiaction_value_changed[IS_LASTONE];

int muiaction_init_done=0;

#define MUIA_Prop_Pressed 0x80422cd7 /* V6 g BOOL */ /* private */

void mgtk_muiaction_init() {
  if(muiaction_init_done) {
    return;
  }

  /* mapping between gtk events (muiaction_event) and mui events (MUIA_events) for widget types (IS_widget)*/
  /* 0 means TODO ;) */

  muiaction_activate[IS_WINDOW]=0;
  muiaction_activate[IS_MENUBAR]=0;
  muiaction_activate[IS_MENU]=0;
  muiaction_activate[IS_OPTIONMENU]=MUIA_Cycle_Active;
  muiaction_activate[IS_MENUITEM]=MUIA_Menuitem_Trigger;
  muiaction_activate[IS_TABLE]=0;
  muiaction_activate[IS_TABLECELL]=0;
  muiaction_activate[IS_ENTRY]=MUIA_String_Acknowledge;
  muiaction_activate[IS_SPINBUTTON]=MUIA_String_Acknowledge;
  muiaction_activate[IS_TOGGLEBUTTON]=0;
//  muiaction_activate[IS_CHECKBUTTON]=0;
  muiaction_activate[IS_BUTTON]=0;

  muiaction_clicked[IS_WINDOW]=0;
  muiaction_clicked[IS_MENUBAR]=0;
  muiaction_clicked[IS_MENU]=0;
  muiaction_clicked[IS_MENUITEM]=0;
  muiaction_clicked[IS_TABLE]=0;
  muiaction_clicked[IS_TABLECELL]=0;
  muiaction_clicked[IS_ENTRY]=0;
  muiaction_clicked[IS_TOGGLEBUTTON]=0;
  //muiaction_clicked[IS_CHECKBUTTON]=MUIA_Selected;
  muiaction_clicked[IS_BUTTON]=MUIA_Pressed;
  muiaction_clicked[IS_ARROW]=MUIA_Pressed;

  muiaction_value_changed[IS_WINDOW]=0;
  muiaction_value_changed[IS_MENUBAR]=0;
  muiaction_value_changed[IS_MENU]=0;
  muiaction_value_changed[IS_MENUITEM]=0;
  muiaction_value_changed[IS_TABLE]=0;
  muiaction_value_changed[IS_TABLECELL]=0;
  muiaction_value_changed[IS_ENTRY]=0;
  muiaction_value_changed[IS_TOGGLEBUTTON]=0;
  //muiaction_value_changed[IS_CHECKBUTTON]=0;
  muiaction_value_changed[IS_BUTTON]=0;
  muiaction_value_changed[IS_LABEL]=0;
  muiaction_value_changed[IS_IMAGE]=0;
  muiaction_value_changed[IS_RADIO]=0;
  muiaction_value_changed[IS_SCROLLBAR]=MUIA_Numeric_Value; 
  muiaction_value_changed[IS_SPINBUTTON]=MA_Spin_String_Value;

/*  muiaction_changed[IS_ENTRY]=MUIA_String_Contents;*/

  muiaction_init_done=1;
}

/*
 * gtk_signal_connect_full is deprecated and should not be used in 
 * newly-written code. Use g_signal_connect_data() instead.
 */
gulong gtk_signal_connect_full  (GtkObject *object,
                                 const gchar *name,
                                 GtkSignalFunc func,
                                 GtkCallbackMarshal unsupported,
                                 gpointer data,
                                 GtkDestroyNotify destroy_func,
                                 gint object_signal,
                                 gint after) {

  return g_signal_connect_data((gpointer) object, name, (GCallback) func, data, (GClosureNotify) destroy_func, (GConnectFlags) object_signal);

}

/*****************************************
 * mgtk_call_destroy_hooks
 *****************************************/
#if 0
void mgtk_call_destroy_hooks(GtkWidget *widget) {
  void (*callme)(GtkWidget *widget,gpointer data);
  mgtkHookList *i;
  mgtkHookList *found=NULL;

  DebOut("mgtk_call_destroy_hooks(%lx)\n",widget);

  i=mgtkhooklist_destroy;

  while(i!=NULL) {
    DebOut("    i->widget=%lx\n",i->widget);
    if(i->widget == widget) {
      found=i;
      DebOut("  found callback: listitem: %lx callback %lx\n",i,i->callback);
      callme=(APTR) found->callback;
      /*call gtk hook */
      DebOut("  calling: %lx(%s)\n",callme,found->data);
      if(found->GConnectFlags == G_CONNECT_SWAPPED) {
        (*callme)(found->data,found->widget);
      }
      else {
        (*callme)(found->widget,found->data);
      }
    }
    i=(APTR) i->next;
  }
  DebOut("  end of scan\n");

  if(found==NULL) {
    printf("WARNING: mgtk_call_destroy_hooks: unknown hook widget (%x)!?\n",(int) widget);
  }
  return;
}
#endif
